#!/usr/bin/env python3
#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
import aiopg
import asyncio
import bcrypt
import markdown
import os.path
import psycopg2
import re
import tornado
import unicodedata

from datetime import datetime, timedelta, date

import logging
import tornado.web
import tornado.locks

from tornado.options import define, options

import fit_api_functions as fit_api_fun

import pandas as pd
import requests
import json

import hashlib
import os


# fit test api
api_base_url =
api_token =

# Define application options
define("port", default=8888, help="run on the given port", type=int)
define("db_host", default="postgres", help="database host")
define("db_port", default=5432, help="database port")
define("db_database", default="lectures", help="database name")
define("db_user", default="user", help="database user")
define("db_password", default="password")

os.environ['TZ'] = 'Europe/Berlin'


class NoResultError(Exception):
    pass

# This function checks if the required tables exist, and if not, it creates them.
async def maybe_create_tables(db):
    try:
        with (await db.cursor()) as cur:
            await cur.execute("SELECT COUNT(*) FROM lecturer LIMIT 1")
            await cur.fetchone()
    except psycopg2.ProgrammingError:
        with open("schema.sql") as f:
            schema = f.read()
        with (await db.cursor()) as cur:
            await cur.execute(schema)
            # Insert admin account
            admin_email = 'admin'
            admin_name = 'Admin'
            admin_role = 'admin'
            admin_password = 'admin'
            admin_password = await tornado.ioloop.IOLoop.current().run_in_executor(
                            None,
                            bcrypt.hashpw,
                            tornado.escape.utf8(admin_password),
                            bcrypt.gensalt(),
                            )
            admin_password = admin_password.decode('utf-8')  # Decode from bytes to string
            await cur.execute("""
                INSERT INTO lecturer (email, name, hashed_password, role)
                VALUES (%s, %s, %s, %s)
            """, (admin_email, admin_name, admin_password, admin_role))
            await db.commit()

# Main application class
class Application(tornado.web.Application):
    def __init__(self, db, fit_api):
        self.db = db
        self.fit_api = fit_api
        self.backend_url = "backend/"
        self.sessions = {} #Here are the current sessions saved
        self.session_timeout = timedelta(minutes=180)  # Session timeout set to 180 minutes
        handlers = [
            (rf"/{self.backend_url}lecturer/create_lecturer", CreateLecturerHandler),
            (rf"/{self.backend_url}lecturer/login-handler", LoginHandler),
            (rf"/{self.backend_url}lecturer/logout-handler", LogoutHandler),
            (rf"/{self.backend_url}lecturer/username", UsernameHandler),
            (rf"/{self.backend_url}create-db-lecture", CreateDbLectureHandler),
            (rf"/{self.backend_url}check-auth", CheckAuthHandler),
            (rf"/{self.backend_url}set-xsrf-token", SetXSRFTokenHandler),
            (rf"/{self.backend_url}sessions", SessionsHandler),
            (rf"/{self.backend_url}courses", CourseHandler),
            (rf"/{self.backend_url}upload-lecture-to-fit", UploadLecturesHandler),
            (rf"/{self.backend_url}get-lectures-to-upload", GetLecturesToUploadHandler),
            (rf"/{self.backend_url}get-courses-of-lecturer", GetCoursesOfLecturerHandler),
            (rf"/{self.backend_url}get-account-details", GetAccountDetailsHandler),
            (rf"/{self.backend_url}edit-account-info", EditAccountInfoHandler),
        ]
        settings = dict(
            lectures_title="AIDHAO Lectures (Tornado Backend)",
            template_path=os.path.join(os.path.dirname(__file__), "templates"),
            static_path=os.path.join(os.path.dirname(__file__), "static"),
            xsrf_cookies=True,
            cookie_secret="__TODO:_GENERATE_YOUR_OWN_RANDOM_VALUE_HERE__",
            login_url=f"/{self.backend_url}lecturer/login",
            debug=True,
        )
        super().__init__(handlers, **settings)
        self.periodic_cleanup()  # Start the periodic cleanup

    # Function to periodically clean up sessions
    def periodic_cleanup(self):
        tornado.ioloop.PeriodicCallback(self.cleanup_sessions, 60000).start()  # Call every 60 seconds

    # Function to clean up expired sessions
    def cleanup_sessions(self):
        now = datetime.now()
        expired_sessions = [sid for sid, session in self.sessions.items() if
                            now - session["last_activity"] > self.session_timeout]
        for sid in expired_sessions:
            del self.sessions[sid]
            logging.info(f"Application || Session {sid} expired and removed.")

# Base handler class for common methods
class BaseHandler(tornado.web.RequestHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    def get_current_user(self):
        session_id = self.get_secure_cookie("session_id")
        if not session_id:
            return None
        return self.application.sessions.get(session_id.decode("utf-8"))

    def row_to_obj(self, row, cur):
        """Convert a SQL row to an object supporting dict and attribute access."""
        obj = tornado.util.ObjectDict()
        for val, desc in zip(row, cur.description):
            obj[desc.name] = val
        return obj

    # Function to execute a SQL statement
    async def execute(self, stmt, args):
        """Execute a SQL statement.

        Must be called with ``await self.execute(...)``
        """
        with (await self.application.db.cursor()) as cur:
            await cur.execute(stmt, args)

    # Function to query for a list of results
    async def query(self, stmt, *args):
        """Query for a list of results.

        Typical usage::

            results = await self.query(...)

        Or::

            for row in await self.query(...)
        """
        with (await self.application.db.cursor()) as cur:
            await cur.execute(stmt, args)
            return [self.row_to_obj(row, cur) for row in await cur.fetchall()]

    # Function to query for exactly one result
    async def queryone(self, stmt, *args):
        """Query for exactly one result.

        Raises NoResultError if there are no results, or ValueError if
        there are more than one.
        """
        results = await self.query(stmt, *args)
        if len(results) == 0:
            raise NoResultError()
        elif len(results) > 1:
            raise ValueError("Expected 1 result, got %d" % len(results))
        return results[0]

    # Function to create a lecture in the database
    async def create_db_lecture(self, data):
        try:
            query = """
                    INSERT INTO lectures (
                        lecturer_id,
                        module_number,                
                        semester, 
                        lecture_id,
                        identifier,
                        workshop_status,
                        title,
                        type,
                        facilities,
                        contact_person,
                        description,
                        description_for_certificate,
                        skill,
                        achievement_scores,
                        deny_download_of_certificates,
                        certificate_blocks,
                        language,
                        data_privacy_of_speakers_allowed,
                        speakers,
                        target_audiences,
                        no_registration_required,
                        registration_via_waiting_list,
                        registration_after_start_allowed,
                        registration_open_for_external,
                        min_seats,
                        max_seats,
                        registration_start_date,
                        registration_end_date,
                        cancellation_end_date,
                        reminder_mail_date,
                        text_for_registration_mail,
                        text_for_reminder_mail,
                        mail_footer,
                        notify_waiting_list,
                        start_date,
                        start_time,
                        end_date,
                        end_time,
                        location_id,
                        location_other,
                        created_at,
                        updated_at)
                    VALUES (
                        -- additional data
                        %(lecturer_id)s,
                        %(module_number)s,
                        %(semester)s,
                        %(lecture_id)s,
                        -- fit data
                        %(identifier)s,
                        %(workshopStatus)s,
                        %(title)s,
                        %(type)s,
                        ARRAY[%(facilities)s]::TEXT[], -- Handle array of text for facilities
                        %(contactPerson)s,
                        %(description)s,
                        %(descriptionForCertificate)s,
                        %(skill)s,
                        %(achievementScores)s,
                        %(denyDownloadOfCertificates)s,
                        ARRAY[%(certificateBlocks)s]::TEXT[], -- Handle array of text for certificate_blocks
                        %(language)s,
                        %(dataPrivacyOfSpeakersAllowed)s,
                        ARRAY[%(speakers)s]::TEXT[], -- Handle array of text for speakers
                        ARRAY[%(targetAudiences)s]::TEXT[], -- Handle array of text for target_audiences
                        %(noRegistrationRequired)s,
                        %(registrationViaWaitingList)s,
                        %(registrationAfterStartAllowed)s,
                        %(registrationOpenForExternal)s,
                        %(minSeats)s,
                        %(maxSeats)s,
                        %(registrationStartDate)s,
                        %(registrationEndDate)s,
                        %(cancellationEndDate)s,
                        %(reminderMailDate)s,
                        %(textForRegistrationMail)s,
                        %(textForReminderMail)s,
                        %(mailFooter)s,
                        %(notifyWaitingList)s,
                        %(startDate)s,
                        %(startTime)s,
                        %(endDate)s,
                        %(endTime)s,
                        %(locationId)s,
                        %(locationOther)s,
                        CURRENT_TIMESTAMP,
                        CURRENT_TIMESTAMP)
                """

            await self.execute(query, data)

        except Exception as e:
            logging.error(f"Error in create_db_lecture: {e}")

    # Function to create a new lecture
    async def create_lecture(self, data):
        query = """
            INSERT INTO lectures (
                lecturer_id,
                slug,
                title,
                type,
                facilities,
                contact_person,
                description,
                description_for_certificate,
                skill,
                achievement_scores,
                deny_download_of_certificates,
                certificate_blocks,
                language,
                data_privacy_of_speakers_allowed,
                speakers,
                target_audiences,
                no_registration_required,
                registration_via_waiting_list,
                registration_after_start_allowed,
                registration_open_for_external,
                min_seats,
                max_seats,
                registration_start_date,
                registration_end_date,
                cancellation_end_date,
                reminder_mail_date,
                text_for_registration_mail,
                text_for_reminder_mail,
                mail_footer,
                notify_waiting_list,
                workshop_status,
                module_number,
                semester,
                identifier,
                created_at,
                updated_at)
            VALUES (
                %(lecturer_id)s,
                %(slug)s,
                %(title)s,
                %(type)s,
                %(facilities)s,
                %(contact_person)s,
                %(description)s,
                %(description_for_certificate)s,
                %(skill)s,
                %(achievement_scores)s,
                %(deny_download_of_certificates)s,
                %(certificate_blocks)s,
                %(language)s,
                %(data_privacy_of_speakers_allowed)s,
                %(speakers)s,
                %(target_audiences)s,
                %(no_registration_required)s,
                %(registration_via_waiting_list)s,
                %(registration_after_start_allowed)s,
                %(registration_open_for_external)s,
                %(min_seats)s,
                %(max_seats)s,
                %(registration_start_date)s,
                %(registration_end_date)s,
                %(cancellation_end_date)s,
                %(reminder_mail_date)s,
                %(text_for_registration_mail)s,
                %(text_for_reminder_mail)s,
                %(mail_footer)s,
                %(notify_waiting_list)s,
                %(workshop_status)s,
                %(module_number)s,
                %(semester)s,
                %(identifier)s,
                CURRENT_TIMESTAMP,
                CURRENT_TIMESTAMP)
        """

        await self.execute(query, data)

    # Function to create a dictionary for the database lecture
    async def lecture_db_dic(self, add_data, fit_data):
        try:
            new_lec = {
                # additional data
                'lecturer_id': add_data.get('lecturer_id', None),
                'module_number': add_data.get('module_number', None),
                'semester': add_data.get('semester', None),
                'lecture_id': add_data.get('lecture_id', None),
                # fit data
                'identifier': fit_data.get('identifier', None),
                'workshopStatus': fit_data.get('workshopStatus', None),
                'title': fit_data.get('title', None),
                'type': fit_data.get('type', None),
                'facilities': fit_data.get('facilities', []),  # Default to an empty list
                'contactPerson': fit_data.get('contactPerson', None),
                'description': fit_data.get('description', None),
                'descriptionForCertificate': fit_data.get('descriptionForCertificate', None),
                'skill': fit_data.get('skill', None),
                'achievementScores': fit_data.get('achievementScores', None),
                'denyDownloadOfCertificates': fit_data.get('denyDownloadOfCertificatesForApplicants', None),
                'certificateBlocks': fit_data.get('certificateBlocks', []),  # Default to an empty list
                'language': fit_data.get('language', None),
                'dataPrivacyOfSpeakersAllowed': fit_data.get('dataPrivacyOfSpeakersAllowed', None),
                'speakers': fit_data.get('speakers', []),  # Default to an empty list
                'targetAudiences': fit_data.get('targetAudiences', []),  # Default to an empty list
                'noRegistrationRequired': fit_data.get('noRegistrationRequired', None),
                'registrationViaWaitingList': fit_data.get('registrationViaWaitingList', None),
                'registrationAfterStartAllowed': fit_data.get('registrationAfterStartAllowed', None),
                'registrationOpenForExternal': fit_data.get('registrationOpenForExternal', None),
                'minSeats': fit_data.get('minSeats', None),
                'maxSeats': fit_data.get('maxSeats', None),
                'registrationStartDate': fit_data.get('registrationStartDate', None),
                'registrationEndDate': fit_data.get('registrationEndDate', None),
                'cancellationEndDate': fit_data.get('cancellationEndDate', None),
                'reminderMailDate': fit_data.get('reminderMailDate', None),
                'textForRegistrationMail': fit_data.get('textForRegistrationMail', None),
                'textForReminderMail': fit_data.get('textForReminderMail', None),
                'mailFooter': fit_data.get('mailFooter', None),
                'notifyWaitingList': fit_data.get('notifyWaitingList', False),
                'startDate': fit_data.get('dates', [{}])[0].get('startDate', None),  # get from sub list dates
                'startTime': fit_data.get('dates', [{}])[0].get('startTime', None),
                'endDate': fit_data.get('dates', [{}])[0].get('endDate', None),
                'endTime': fit_data.get('dates', [{}])[0].get('endTime', None),
                'locationId': fit_data.get('dates', [{}])[0].get('locationId', None),
                'locationOther': fit_data.get('dates', [{}])[0].get('locationOther', None),
            }

            return new_lec

        except Exception as e:
            logging.error(f"Error in lecture_db_dic: {e}")

# Handler to set XSRF token
class SetXSRFTokenHandler(BaseHandler):
    def get(self):
        self.xsrf_token  # Ensure the XSRF token is set
        self.write({"message": "XSRF token set"})

# Handler to get all active sessions
class SessionsHandler(BaseHandler):
    def get(self):
        logging.info(f'All active sessions: {self.application.sessions}')
        sessions_serializable = {sid: {**session, "last_activity": session["last_activity"].isoformat()}
                                 for sid, session in self.application.sessions.items()}
        self.write({"sessions": sessions_serializable})

# Handler to get courses
class CourseHandler(BaseHandler):
    def get(self):
        fit_api = self.application.fit_api
        fit_lectures = fit_api.get_lectures(status="DRAFT")
        self.write({"courses": json.dumps(fit_lectures)})

# Handler for user login
class LoginHandler(BaseHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    async def post(self):
        data = tornado.escape.json_decode(self.request.body)
        logging.info(f"Received data: {data}")
        #Checking if password and email are receved
        if not data['email'] or not data['password']:
            logging.info("Received login data is empty")
            self.write({"success": False, "Message": "Login data is empty"})
            return
        #Get a list of all entries with the email X
        query = "SELECT * FROM lecturer WHERE email = %s"
        existing_lecturer = await self.query(query, (data['email']))
        logging.info(f'Login Handler: Existing Lecturer Status -> {existing_lecturer}')

        #Check if the list has an entry
        if existing_lecturer:
            logging.info(f'Login Handler || Lecturer exists')
            #Get password
            query = "SELECT hashed_password FROM lecturer WHERE email = %s"
            result = await self.queryone(query, data['email'])
            hashed_password_user = result['hashed_password']
            #Get Username
            query = "SELECT name FROM lecturer WHERE email = %s"
            result = await self.queryone(query, data['email'])
            name_user = result['name']
            #Get User role
            query = "SELECT role FROM lecturer WHERE email = %s"
            result = await self.queryone(query, data['email'])
            role_user = result['role']
            #Check passwords
            password_equal = await tornado.ioloop.IOLoop.current().run_in_executor(
                None,
                bcrypt.checkpw,
                tornado.escape.utf8(data["password"]),
                tornado.escape.utf8(hashed_password_user),
            )
            # Get User id
            query = "SELECT id FROM lecturer WHERE email = %s"
            result = await self.queryone(query, data['email'])
            user_id = result['id']
            #If password are equal set a session and a secure cookie
            if password_equal:
                session_id = str(uuid.uuid4())
                self.application.sessions[session_id] = {
                    "email": data['email'],
                    "name": name_user,
                    "last_activity": datetime.now(),
                    "role": role_user,
                    "id": user_id
                }
                self.set_secure_cookie("session_id", session_id, httponly=True, secure=True)
                self.write({"success": True}) #Send success to the frontend
                logging.info('Login Handler || Successful Login')
            else:
                logging.info('Login Handler || Password is incorrect')
                self.write({"success": False, "message": "Invalid credentials"}) #Send fail to the frontend
        else:
            logging.info(f'Login Handler || Lecturer does not exist')
            self.write({"success": False, "message": "Invalid credentials"})

# Handler for user logout
class LogoutHandler(BaseHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    def get(self):
        session_id = self.get_secure_cookie("session_id")
        success = False
        #If session exist it will be deleted
        if session_id:
            self.application.sessions.pop(session_id.decode(), None)
            self.clear_cookie("session_id")
            success = True

        self.write({"success": success}) #Success message to frontend

# Handler to get the username of the logged-in user
class UsernameHandler(BaseHandler):
    def get(self):
        session_id = self.get_secure_cookie("session_id")
        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        logging.info(f'Protected Handler || User_session: {user_session}')
        #Send the name and role to the frontend -> for the header
        if user_session:
            name = user_session.get("name")
            email = user_session.get("email")
            self.write({"name": name, "role": user_session.get('role')})

        else:
            self.write({"message": "No user session found."})

# Function to validate user data
def validate_user_data(data):
    errors = []

    if not data.get('password'):
        errors.append("Password is required.")
    if not data.get('email'):
        errors.append("Email is required.")
    if not data.get('name'):
        errors.append("Name is required.")

    email_regex = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
    if data.get('email') and not re.match(email_regex, data['email']):
        errors.append("Invalid email format.")

    name_regex = r'^[a-zA-Z\s]+$'
    if data.get('name') and not re.match(name_regex, data['name']):
        errors.append("Name must not contain special characters.")

    return errors

# Handler to create a new lecturer
class CreateLecturerHandler(BaseHandler):

    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    async def post(self):
        data = tornado.escape.json_decode(self.request.body)
        logging.info(f"CreateLecturerHandler || Received data: {data}")
        validation_errors = validate_user_data(data)
        #If there is an error with the userdata -> fail message to the frontend
        if validation_errors:
            self.write({'success': False, 'message': validation_errors})
            return

        hashed_password = await tornado.ioloop.IOLoop.current().run_in_executor(
            None,
            bcrypt.hashpw,
            tornado.escape.utf8(data['password']),
            bcrypt.gensalt(),
        )

        #Check if lecturer allready exists
        query = """
            SELECT * FROM lecturer WHERE email = %s
        """
        existing_lecturer = await self.query(query, (data['email']))
        logging.info(f'Create Lecturer: {existing_lecturer}')
        if existing_lecturer:
            self.write({'success': False, 'message': 'User already exists'})
            return
        #Create lecturer
        lecturer = await self.queryone(
            "INSERT INTO lecturer (email, name, hashed_password, role) "
            "VALUES (%s, %s, %s, %s) RETURNING id",
            data['email'],
            data['name'],
            tornado.escape.to_unicode(hashed_password),
            'lecturer',
        )
        self.write({'success': True}) #success message for frontend
        logging.info(f'Create Lecturer: {lecturer} successfully inserted')

# Handler to get account details of the logged-in user
class GetAccountDetailsHandler(BaseHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    async def get(self):
        session_id = self.get_secure_cookie("session_id")
        #Check if user is authorised
        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        if not user_session:
            self.set_status(401)
            self.write({"message": "Unauthorized"})
            return

        try:
            lecturer_id = user_session.get('id')
            logging.info(f'GetCoursesOfLecturer | Lecturer_id: {lecturer_id}')
            account_info = await self.query("SELECT * FROM lecturer WHERE id = %s", lecturer_id) #Send Account info to the frontend
            self.write(json.dumps({"account_info": account_info}))
        except Exception as e:
            logging.error(f"Error fetching account details: {e}")
            self.set_status(500)
            self.write({"message": "Internal server error"})

# Handler to edit account information
class EditAccountInfoHandler(BaseHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    async def post(self):
        session_id = self.get_secure_cookie("session_id")
        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        logging.info(f'EditAccountInfo | sessionID {user_session}')
        #Check if user is authorized
        if not user_session:
            self.set_status(401)
            self.write({"success": "False", "message": "Unauthorized"})
            return
        try:
            #Get User data and update it
            lecturer_id = user_session.get('id')
            data = tornado.escape.json_decode(self.request.body)
            logging.info(f"Received data: {data}")
            email = data.get('email')
            name = data.get('name')
            hashed_password = data.get('password')

            query = "SELECT hashed_password FROM lecturer WHERE id = %s"
            result = await self.queryone(query, lecturer_id)
            hashed_password_user = result['hashed_password']
            logging.info(f'EditAccountInfo | Password {hashed_password_user}')

            password_equal = await tornado.ioloop.IOLoop.current().run_in_executor(
                None,
                bcrypt.checkpw,
                tornado.escape.utf8(hashed_password),
                tornado.escape.utf8(hashed_password_user),
            )

            logging.info(f'EditAccountInfo | password_equal: {password_equal}')
            if password_equal:
                logging.info(f'--------------Test before SQL')
                update_query = """
                     UPDATE lecturer 
                     SET email = %s, name = %s
                     WHERE id = %s
                     RETURNING id
                 """
                lecturer = await self.query(update_query, email, name, lecturer_id)
                self.write({"success": "True"})
        except Exception as e:
            logging.error(f"Error fetching account details: {e}")
            self.set_status(500)
            self.write({"success": "False", "message": "Internal server error"})

# Function to get the data structure for the database
def getDbDataStructure():
    form_data = {
        "module_number": None,
        "semester": None,
        "identifier": None,
        "workshopStatus": "DRAFT",
        "title": None,
        "type": None,
        "facilities": ["911F4794-77B9-A08A-1835-302DF75C74B8"],
        "contactPerson": "intern@la2.de",
        "description": None,
        "descriptionForCertificate": "",
        "skill": None,
        "achievementScores": 6,
        "denyDownloadOfCertificatesForApplicants": False,
        "certificateBlocks": ["35A35E1E-C732-157D-6D99-D916E4A138F2"],
        "language": "EN",
        "dataPrivacyOfSpeakersAllowed": True,
        "speakers": "csl-referent@la2.de",
        "targetAudiences": ["9F1CF175-BBCA-B469-75E5-7409B048DD53"],
        "noRegistrationRequired": False,
        "registrationViaWaitingList": False,
        "registrationAfterStartAllowed": True,
        "registrationOpenForExternal": False,
        "minSeats": 0,
        "maxSeats": 0,
        "registrationStartDate": "2023-12-24",
        "registrationEndDate": "2023-12-25",
        "cancellationEndDate": "2023-12-25",
        "reminderMailDate": "2023-12-25",
        "textForRegistrationMail": "",
        "textForReminderMail": "",
        "mailFooter": "",
        "startDate": None,
        "endDate": None,
        "notifyWaitingList": True,
        "locationId": None,
        "locationOther": None,
        "dates": [
            {
                "startDate": None,
                "endDate": None,
                "notifyWaitingList": None,
                "locationId": None,
                "locationOther": None
            },
        ],
    }
    return form_data

# Handler to check authentication
class CheckAuthHandler(BaseHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    def get(self):
        self.xsrf_token  # Ensure the XSRF token is set

    async def post(self):
        #Check if user has the right role
        session_id = self.get_secure_cookie("session_id")
        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        data = tornado.escape.json_decode(self.request.body)
        logging.info(f'{data}')
        if data == 'all':
            if not user_session:
                self.write({"auth": False})
            else:
                self.write({"auth": True})
        elif data == 'admin':
            if not user_session.get('role') == 'admin':
                self.write({"auth": False})
            else:
                self.write({"auth": True})

# Handler to create a new lecture in the database
class CreateDbLectureHandler(BaseHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with, content-type, X-XSRFToken")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")

    def options(self):
        self.set_status(204)
        self.finish()

    def get(self):
        self.xsrf_token  # Ensure the XSRF token is set

    async def post(self):

        session_id = self.get_secure_cookie("session_id")
        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        lecturer_id = user_session.get('id')
        #Check if user is authorized
        if not user_session:
            self.set_status(401)
            self.write({"success": "False", "message": "Unauthorized"})
            return

        try:
            logging.info(f"XSRF-Cookie: {self.xsrf_token}")
            self.check_xsrf_cookie()  # Check the XSRF token
            logging.info("XSRF token successfully verified")

            form_data = getDbDataStructure()

            data = tornado.escape.json_decode(self.request.body)
            logging.info(f"CreateDbLectureHandler || Received data: {data}")
            module_number = data.get('module_number')
            semester = data.get('semester')
            lecture_id = f"{module_number}-{semester}" #Create lecture id

            add_data = {
                'lecture_id': lecture_id, # data from the frontend
                'lecturer_id': int(lecturer_id),
                'module_number': module_number, # data from the frontend
                'semester': semester # data from the frontend
            }
            logging.info(f"Additional data: {add_data}")
            title = data.get('title')
            title = module_number + ": " + title + " " + semester #Create Modul title
            fit_data = {
                'identifier': form_data.get('identifier', None),
                'workshopStatus': form_data.get('workshopStatus', 'DRAFT'),
                'title': title, # data from the frontend
                'type': form_data.get('type', None),
                'facilities': form_data.get('facilities'),
                'contactPerson': form_data.get('contactPerson', None),
                'description': data.get('description'), # data from the frontend
                'descriptionForCertificate': form_data.get('descriptionForCertificate', None),
                'skill': form_data.get('skill'),
                'achievementScores': form_data.get('achievementScores'),
                'denyDownloadOfCertificatesForApplicants': form_data.get('denyDownloadOfCertificatesForApplicants',
                                                                         None),
                'certificateBlocks': form_data.get('certificateBlocks', []),
                'language': form_data.get('language', None),
                'dataPrivacyOfSpeakersAllowed': form_data.get('dataPrivacyOfSpeakersAllowed', None),
                'speakers': form_data.get('speakers'),
                'targetAudiences': form_data.get('targetAudiences'),
                'noRegistrationRequired': form_data.get('noRegistrationRequired', None),
                'registrationViaWaitingList': form_data.get('registrationViaWaitingList', None),
                'registrationAfterStartAllowed': form_data.get('registrationAfterStartAllowed', None),
                'registrationOpenForExternal': form_data.get('registrationOpenForExternal', None),
                'minSeats': form_data.get('minSeats'),
                'maxSeats': form_data.get('maxSeats'),
                'registrationStartDate': form_data.get('registrationStartDate', None),
                'registrationEndDate': form_data.get('registrationEndDate', None),
                'cancellationEndDate': form_data.get('cancellationEndDate', None),
                'reminderMailDate': form_data.get('reminderMailDate', None),
                'textForRegistrationMail': form_data.get('textForRegistrationMail', None),
                'textForReminderMail': form_data.get('textForReminderMail', None),
                'mailFooter': form_data.get('mailFooter', None),
                'notifyWaitingList': form_data.get('notifyWaitingList', None),
                'dates': [
                    {
                        'startDate': data.get('startDate', None), # data from the frontend
                        'startTime': data.get('startTime', None),
                        'endDate': data.get('endDate', None), # data from the frontend
                        'endTime': data.get('endTime', None)
                    }
                ]
            }
            logging.info(f"Fit data: {fit_data}")

            new_lec = await self.lecture_db_dic(add_data, fit_data) #Create a dic from both data sets
            logging.info(f"All data: {new_lec}")
            await self.create_db_lecture(new_lec) #Create entry
            self.write({"success": True, "message": "A new course has been added. Look under 'Your courses'"})
        except Exception as e:
            self.write({"success": False, "message": "An error occurred"})
            logging.info(f"Something went wrong")

class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (datetime, date)):
            return obj.isoformat()
        return super(DateTimeEncoder, self).default(obj)

# Handler to get courses of the logged-in lecturer
class GetCoursesOfLecturerHandler(BaseHandler):
    async def get(self):
        session_id = self.get_secure_cookie("session_id")

        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        if not user_session:
            self.set_status(401)
            self.write({"message": "Unauthorized"})
            return

        try:
            #Get courses that are created by a special user
            lecturer_id = user_session.get('id')
            logging.info(f'GetCoursesOfLecturer | Lecturer_id: {lecturer_id}')
            lectures = await self.query("SELECT * FROM lectures WHERE lecturer_id = %s", lecturer_id)
            if not lectures:
                self.write({"message": "No lectures"})
                return

            lectures_list = [dict(lecture) for lecture in lectures]
            logging.info(f"GetLecturesToUploadHandler | lectures data: {lectures_list}")
            self.write(json.dumps({"lectures": lectures_list}, cls=DateTimeEncoder))
        except Exception as e:
            logging.error(f"Error fetching lectures: {e}")
            self.set_status(500)
            self.write({"message": "Internal server error"})

# Handler to get lectures that need to be uploaded
class GetLecturesToUploadHandler(BaseHandler):
    async def get(self):
        try:
            #Get courses with no identifier
            db_lectures = await self.query("SELECT * FROM lectures WHERE identifier IS NULL")
            if not db_lectures:
                self.write({"message": "No lectures with NULL identifier found"})
                return

            lectures_list = [dict(lecture) for lecture in db_lectures]
            logging.info(f"GetLecturesToUploadHandler | lectures data: {lectures_list}")
            self.write(json.dumps({"lectures": lectures_list}, cls=DateTimeEncoder))
        except Exception as e:
            logging.error(f"Error fetching lectures: {e}")
            self.set_status(500)
            self.write({"message": "Internal server error"})

# Handler to upload lectures to the FIT platform
class UploadLecturesHandler(BaseHandler):

    async def post(self):

        session_id = self.get_secure_cookie("session_id")
        user_session = self.application.sessions.get(session_id.decode("utf-8"))
        if not user_session:
            self.set_status(401)
            self.write({"message": "Unauthorized"})
            return

        ids = tornado.escape.json_decode(self.request.body)
        ids = ids.get('selectedCourses', [])
        logging.info(f"UploadLecturesHandler | Received data from frontend: {ids}")

        db_lectures = await self.query("SELECT * FROM lectures WHERE id = ANY(%s)", (ids,)) #Get courses with the ids selected in the frontend
        logging.info(f"db lectures data: {db_lectures}")

        fit_api = self.application.fit_api

        for db_lecture in db_lectures:

            try:
                fit_data = {
                    'workshopStatus': db_lecture['workshop_status'],
                    'title': db_lecture['title'],
                    'type': db_lecture['type'],
                    'facilities': db_lecture['facilities'][0],
                    'contactPerson': db_lecture['contact_person'],
                    'description': db_lecture['description'],
                    'descriptionForCertificate': db_lecture['description_for_certificate'],
                    'skill': db_lecture['skill'],
                    'achievementScores': db_lecture['achievement_scores'],
                    'denyDownloadOfCertificatesForApplicants': db_lecture['deny_download_of_certificates'],
                    'certificateBlocks': db_lecture['certificate_blocks'][0],
                    'language': db_lecture['language'],
                    'dataPrivacyOfSpeakersAllowed': db_lecture['data_privacy_of_speakers_allowed'],
                    'speakers': db_lecture['speakers'],
                    'targetAudiences': db_lecture['target_audiences'][0],
                    'noRegistrationRequired': db_lecture['no_registration_required'],
                    'registrationViaWaitingList': db_lecture['registration_via_waiting_list'],
                    'registrationAfterStartAllowed': db_lecture['registration_after_start_allowed'],
                    'registrationOpenForExternal': db_lecture['registration_open_for_external'],
                    'minSeats': str(int(db_lecture['min_seats'])),
                    'maxSeats': str(int(db_lecture['max_seats'])),
                    'registrationStartDate': db_lecture['registration_start_date'].strftime('%Y-%m-%d') if db_lecture[
                        'registration_start_date'] else None,
                    'registrationEndDate': db_lecture['registration_end_date'].strftime('%Y-%m-%d') if db_lecture[
                        'registration_end_date'] else None,
                    'cancellationEndDate': db_lecture['cancellation_end_date'].strftime('%Y-%m-%d') if db_lecture[
                        'cancellation_end_date'] else None,
                    'reminderMailDate': db_lecture['reminder_mail_date'].strftime('%Y-%m-%d') if db_lecture[
                        'reminder_mail_date'] else None,
                    'textForRegistrationMail': db_lecture['text_for_registration_mail'],
                    'textForReminderMail': db_lecture['text_for_reminder_mail'],
                    'mailFooter': db_lecture['mail_footer'],
                    'notifyWaitingList': db_lecture['notify_waiting_list'],
                    'dates': [
                        {
                            'startDate': db_lecture['start_date'].strftime('%Y-%m-%d') if db_lecture[
                                'start_date'] else None,
                            'startTime': db_lecture['start_time'],
                            'endDate': db_lecture['end_date'].strftime('%Y-%m-%d') if db_lecture['end_date'] else None,
                            'endTime': db_lecture['end_time'],
                            'locationId': db_lecture['location_id'],
                            'locationOther': db_lecture['location_other']
                        }
                    ]
                }
                logging.info(f"data to be uploaded to fit before: {fit_data}")
                fit_data = fit_api_fun.clean_dict_values(fit_data)
                logging.info(f"data to be uploaded to fit after: {fit_data}")
            except Exception as e:
                logging.error(f"Error preparing data for uploading lecture to FIT platform: {e}")
                continue

            try:
                response = fit_api.create_lecture(fit_data) #Create lecture in fit
                logging.error(f'UploadLecturesHandler | Response {response}')
                #Set the identifier in the lectures database
                data = {
                    'new_identifier': response['identifier'],
                    'lecture_id': db_lecture['lecture_id']
                }
                query = """
                    UPDATE lectures SET
                        identifier = %(new_identifier)s,
                        updated_at = CURRENT_TIMESTAMP
                    WHERE lecture_id = %(lecture_id)s
                """
                await self.execute(query, data)

            except Exception as e:
                logging.error(f"Error uploading lecture to FIT platform: {e}")
                self.write({"success": False, "message": f"Error: {e}"})
                continue

            self.write({"success": True, "message": f"Successfully uploaded {len(db_lectures)} lectures"})

# Function to create a database connection pool
async def create_db_pool():
    retries = 10
    for attempt in range(retries):
        try:
            db_pool = await aiopg.create_pool(
                host=options.db_host,
                port=options.db_port,
                user=options.db_user,
                password=options.db_password,
                dbname=options.db_database,
            )
            await maybe_create_tables(db_pool)
            logging.info(f"Database connection pool created successfully on attempt {attempt + 1}")
            return db_pool
        except (Exception, psycopg2.OperationalError) as e:
            logging.error(f"Failed to connect to the database, retrying... ({attempt + 1}/{retries})")
            logging.error("Error: %s", e)
            await asyncio.sleep(5)  # wait for 5 seconds before retrying
    raise RuntimeError("Database connection could not be established after several retries.")

# Main function to start the application
async def main():
    tornado.options.parse_command_line()

    db_pool = await create_db_pool()
    fit_api_inst = fit_api_fun.fit_api(api_base_url, api_token)

    app = Application(db_pool, fit_api_inst)
    app.listen(options.port)
    logging.info(f"Listening on http://localhost:{options.port}/backend")

    shutdown_event = tornado.locks.Event()
    await shutdown_event.wait()
    logging.info("Server shutdown gracefully.")

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    tornado.ioloop.IOLoop.current().run_sync(main)
