import logging

import requests
import json
import pandas as pd
import numpy as np


def extract_modules(json_tree, only_name_and_id=True):
    modules = []
    def traverse(node):
        # Check if the node is a leaf (has no 'blocks' or 'blocks' is None)
        if node.get('blocks') is None and node.get('parent') is not None:
            modules.append(node)
        else:
            if node.get('blocks') is None:
                return
            else:
                # If it's not a leaf and has 'blocks', traverse each of the blocks recursively
                for block in node.get('blocks', []):
                    traverse(block)
    
    # Start the traversal with the initial JSON tree
    for item in json_tree:
        traverse(item)

    if only_name_and_id:    
        return [[
                    module['name'], 
                    module['identifier'] 
         ] for module in modules]
    else:
        return modules

# Function to lookup identifier by lecture name
def get_module_identifier(lectures_list,lecture_name):
    lectures_df = pd.DataFrame(lectures_list, columns=['Lecture Name', 'Identifier'])
    filtered_df = lectures_df[lectures_df['Lecture Name'] == lecture_name]
    if not filtered_df.empty:
        return filtered_df.iloc[0]['Identifier']
    else:
        return "Lecture name not found"


def clean_value(value):
    """Recursively clean the given value."""
    if isinstance(value, float) and (pd.isna(value) or value in [np.inf, -np.inf]):
        return None  # Skip NaN, inf, -inf by returning None
    elif isinstance(value, dict):
        return clean_dict_values(value)  # Recursively clean dictionaries
    elif isinstance(value, list):
        # Recursively clean each item in the list, skipping None values
        return [clean_value(item) for item in value if clean_value(item) is not None]
    else:
        return value


def clean_dict_values(lec_dic):
    """Clean the given dictionary recursively."""
    cleaned_dict = {}
    for key, value in lec_dic.items():
        cleaned_value = clean_value(value)
        if cleaned_value is not None:  # Only add if the cleaned value is not None
            cleaned_dict[key] = cleaned_value
    return cleaned_dict


def lecture_json(lec_dic):
    # Construct the JSON object
    try:
        # Log den Inhalt von lec_dic
        logging.info(f"lec_dic content: {lec_dic}")

        # Construct the JSON object
        new_lecture = {
            "workshopStatus": lec_dic["workshopStatus"],
            "title": lec_dic["title"],
            "type": lec_dic["type"],
            "facilities": lec_dic["facilities"].split('/'),
            "contactPerson": lec_dic["contactPerson"],
            "description": lec_dic["description"],
            "descriptionForCertificate": lec_dic["descriptionForCertificate"],
            "skill": lec_dic["skill"],
            "achievementScores": str(lec_dic["achievementScores"]),
            "denyDownloadOfCertificatesForApplicants": str(lec_dic["denyDownloadOfCertificatesForApplicants"]),
            "certificateBlocks": lec_dic["certificateBlocks"].split('/'),
            "language": lec_dic["language"],
            "dataPrivacyOfSpeakersAllowed": str(lec_dic["dataPrivacyOfSpeakersAllowed"]),
            "speakers": lec_dic["speakers"].split('/') if not pd.isna(lec_dic["speakers"]) else None,
            "targetAudiences": str(lec_dic["targetAudiences"]).split('/') if lec_dic["targetAudiences"] not in [None,
                                                                                                                ''] else None,
            "noRegistrationRequired": str(lec_dic["noRegistrationRequired"]),
            "registrationViaWaitingList": str(lec_dic["registrationViaWaitingList"]),
            "registrationAfterStartAllowed": lec_dic["registrationAfterStartAllowed"],
            "registrationOpenForExternal": str(lec_dic["registrationOpenForExternal"]),
            "minSeats": str(int(lec_dic["minSeats"])),
            "maxSeats": str(int(lec_dic["maxSeats"])),
            "registrationStartDate": lec_dic["registrationStartDate"],
            "registrationEndDate": lec_dic["registrationEndDate"],
            "cancellationEndDate": lec_dic["cancellationEndDate"],
            "reminderMailDate": lec_dic["reminderMailDate"],
            "textForRegistrationMail": lec_dic["textForRegistrationMail"],
            "textForReminderMail": lec_dic["textForReminderMail"],
            "mailFooter": lec_dic["mailFooter"],
            "notifyWaitingList": str(lec_dic["notifyWaitingList"]),
            "dates": [
                {
                    "startDate": lec_dic["startDate"],
                    "startTime": lec_dic["startTime"],
                    "endDate": lec_dic["endDate"],
                    "endTime": lec_dic["endTime"],
                    "locationId": lec_dic["locationId"],
                    "locationOther": lec_dic["locationOther"]
                }
            ]
        }

        # Clean the dictionary values
        new_lecture = clean_dict_values(new_lecture)
        return new_lecture

    except KeyError as e:
        logging.error(f"KeyError: Missing key {e} in lec_dic")
        raise
    except TypeError as e:
        logging.error(f"TypeError: {e}")
        raise
    except Exception as e:
        logging.error(f"An error occurred: {e}")
        raise


class fit_api:

    def __init__(self, api_base_url, api_token):
        self.api_base_url = api_base_url
        self.api_token = api_token
        self.headers = headers = { 
            "Authorization": f"Bearer {api_token}"
        }
    
    def get_certificates(self):
        try:
            # get participant for certificate
            response = requests.get(
                        str(self.api_base_url+"/certificates"), 
                        headers=self.headers
                        )
            if response.status_code == 200:
                # Successful request
                data = response.json()
                return(data) 
            else:
                # Failed request
                raise Exception("Error with status code:", response.status_code, response.text)
        except requests.ConnectionError:
            print("Failed to connect to the API.")
        except requests.Timeout:
            print("The request timed out.")
        except requests.HTTPError as http_err:
            print(f"HTTP error occurred: {http_err}")
        except json.JSONDecodeError:
            print("Failed to decode JSON from the response.")
        except Exception as err:
            # This catches other exceptions not caught by the specific ones above
            print(f"An error occurred: {err}")

    def get_aidaho_identifier(self):
        certificates = self.get_certificates()
        aidaho = [cert for cert in certificates if cert["name"] == "AIDAHO"]
        return(aidaho[0]["identifier"])
    
    def get_lectures(self,status=None):
        try:
            # get participant for certificate
            if status is not None:
                response = requests.get(
                        str(self.api_base_url+"/workshops?status="+status), 
                        headers=self.headers
                        )
            else:
                response = requests.get(
                        str(self.api_base_url+"/workshops"), 
                        headers=self.headers
                        )
            if response.status_code == 200:
                # Successful request
                data = response.json()
                return(data) 
            else:
                # Failed request
                raise Exception("Error with status code:", response.status_code, response.text)
        except requests.ConnectionError:
            print("Failed to connect to the API.")
        except requests.Timeout:
            print("The request timed out.")
        except requests.HTTPError as http_err:
            print(f"HTTP error occurred: {http_err}")
        except json.JSONDecodeError:
            print("Failed to decode JSON from the response.")
        except Exception as err:
            # This catches other exceptions not caught by the specific ones above
            print(f"An error occurred: {err}")
    
    def get_lecturers_emails(self,lectures,status="REGISTRATION"):
        active_course = [ course for course in lectures if course["workshopStatus"] == status ]

        flattened_list = [speaker for sublist in [course["speakers"] for course in active_course] for speaker in sublist]
        return(flattened_list)

    def create_lecture(self, lec_dic):
        try:
            # Logging des zu sendenden Objekts
            logging.info(f"Creating lecture with payload: {lec_dic}")

            # POST-Anfrage an die API senden
            response = requests.post(
                f"{self.api_base_url}/workshops",
                headers=self.headers,
                json=lec_dic
            )
            # Überprüfen des Statuscodes der Antwort
            if response.status_code == 200:
                data = response.json()
                logging.info(f"Lecture created successfully: {data}")
                return data
            else:
                logging.error(
                    f"Error with status code: {response.status_code}, Response text: {response.text}")
                raise Exception("Error with status code:", response.status_code, response.text)
        except requests.ConnectionError:
            logging.info("Failed to connect to the API.")
        except requests.Timeout:
            logging.info("The request timed out.")
        except requests.HTTPError as http_err:
            logging.info(f"HTTP error occurred: {http_err}")
        except json.JSONDecodeError:
            logging.info("Failed to decode JSON from the response.")
        except ValueError as val_err:
            logging.info(f"Value error: {val_err}")
        except Exception as err:
            logging.info(f"An error occurred: {err}")
    def update_lecture(self,lec_dic):
        try:
            # Construct the JSON object
            new_lecture = lecture_json(lec_dic)
            # Update the lecture
            response = requests.put(
                str(self.api_base_url+"/workshops/"+lec_dic["identifier"]), 
                headers=self.headers,
                json=new_lecture
            )
            if response.status_code == 200:
                # Successful request
                data = response.json()
                return(data) 
            else:
                # Failed request
                raise Exception("Error with status code:", response.status_code, response.text)
        except requests.ConnectionError:
            print("Failed to connect to the API.")
        except requests.Timeout:
            print("The request timed out.")
        except requests.HTTPError as http_err:
            print(f"HTTP error occurred: {http_err}")
        except json.JSONDecodeError:
            print("Failed to decode JSON from the response.")
        except Exception as err:
            # This catches other exceptions not caught by the specific ones above
            print(f"An error occurred: {err}")

    def get_particpants(self,certificate_identifier):
        return False

