import React, { useState, useEffect } from 'react';
import 'bootstrap/dist/css/bootstrap.min.css';

const AccountPage = () => {
  // State to hold account details
  const [accountDetails, setAccountDetails] = useState({
    name: '',
    email: '',
    password: '', // initial password is empty
  });
  // State to hold success and error messages
  const [successMessage, setSuccessMessage] = useState('');
  const [errorMessage, setErrorMessage] = useState('');

  // Function to get a specific cookie value
  function getCookie(name) {
    const value = `; ${document.cookie}`;
    const parts = value.split(`; ${name}=`);
    if (parts.length === 2) return parts.pop().split(';').shift();
    return null;
  }

  // Function to fetch account details from the backend
  const fetchAccountDetails = async () => {
    try {
      const response = await fetch('/backend/get-account-details', {
        method: 'GET',
        credentials: 'include',
      });
      if (!response.ok) {
        const errorText = await response.text();
        throw new Error(`Network response was not ok: ${errorText}`);
      }
      const data = await response.json();
      console.log('Response: ', data);
      const account = data.account_info[0];
      setAccountDetails({
        name: account.name || '',
        email: account.email || '',
        password: '', // we don't fetch the password for security reasons
      });
    } catch (error) {
      console.error('There was a problem with the fetch operation:', error);
    }
  };

  // Function to check if the user is authorized
  const CheckAuth = async () => {
    const xsrfToken = getCookie('_xsrf');
    const check = 'admin'
    try {
      const response = await fetch('/backend/check-auth', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken,
        },
        body: JSON.stringify(check),
      });

      const result = await response.json();
      console.log(result.auth)
      if (response.ok && result.auth){
        console.log('Authorized')
      } else {
        navigate('/unauth');
      }
    } catch (error) {
      console.error('Error setting XSRF token:', error);
    }
  };

  // useEffect hook to fetch account details and check authorization when the component mounts
  useEffect(() => {
    CheckAuth();
    fetchAccountDetails();
  }, []);

  // Function to handle input changes
  const handleChange = (e) => {
    const { name, value } = e.target;
    setAccountDetails((prevDetails) => ({
      ...prevDetails,
      [name]: value,
    }));
  };

  // Function to handle account information edit
  const handleEdit = async () => {
    const xsrfToken = getCookie('_xsrf');
    const { email, name , password } = accountDetails;
    try {
      const response = await fetch('/backend/edit-account-info', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken,
        },
        credentials: 'include',
        body: JSON.stringify({ email, name, password }),
      });
      const result = await response.json();
      console.log(result);
      if (response.ok && result.success) {
        setSuccessMessage('Account information updated successfully!');
        setErrorMessage('');
      } else {
        setErrorMessage(result.message || 'Update failed. Please try again.');
        setSuccessMessage('');
      }
    } catch (error) {
      setErrorMessage('An error occurred. Please try again.');
      setSuccessMessage('');
    }
  };

  return (
    <div className="container mt-5">
      <div className="row justify-content-md-center">
        <div className="col-md-6">
          <h3 className="mb-4">Your Account</h3>
          {successMessage && <div className="alert alert-success">{successMessage}</div>}
          {errorMessage && <div className="alert alert-danger">{errorMessage}</div>}
          <form>
            <div className="mb-3">
              <label htmlFor="formName" className="form-label">Name</label>
              <input
                type="text"
                id="formName"
                name="name"
                className="form-control"
                value={accountDetails.name}
                onChange={handleChange}
              />
            </div>

            <div className="mb-3">
              <label htmlFor="formEmail" className="form-label">Email</label>
              <input
                type="email"
                id="formEmail"
                name="email"
                className="form-control"
                value={accountDetails.email}
                onChange={handleChange}
              />
            </div>

            <div className="mb-3">
              <label htmlFor="formPassword" className="form-label">Password</label>
              <input
                type="password"
                id="formPassword"
                name="password"
                className="form-control"
                value={accountDetails.password}
                onChange={handleChange}
              />
            </div>

            <button type="button" onClick={handleEdit} className="btn btn-primary">
              Edit
            </button>
          </form>
        </div>
      </div>
    </div>
  );
};

export default AccountPage;
