import React, { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';

const FormComponent = () => {
  // State to hold form data
  const [formData, setFormData] = useState({
    title: '',
    module_number: '',
    semester: '',
    description: '',
    startDate: '',
    endDate: '',
    minSeats: 0,
    maxSeats: 0,
  });

  // State to hold success or error messages
  const [message, setMessage] = useState('');
  const navigate = useNavigate();

  // Function to handle input changes
  const handleChange = (e) => {
    const { name, value } = e.target;
    setFormData({
      ...formData,
      [name]: value,
    });
  };

  // Function to get a specific cookie value
  function getCookie(name) {
    const value = `; ${document.cookie}`;
    const parts = value.split(`; ${name}=`);
    if (parts.length === 2) return parts.pop().split(';').shift();
    return null;
  }

  // Function to fetch XSRF token from the backend
  const fetchXsrfToken = async () => {
    try {
      const response = await fetch('/backend/set-xsrf-token', {
        method: 'GET',
        credentials: 'include',
      });

      if (!response.ok) {
        throw new Error('Failed to set XSRF token');
      }

      console.log('XSRF token set');
    } catch (error) {
      console.error('Error setting XSRF token:', error);
    }
  };

  // Function to check if the user is authorized
  const CheckAuth = async () => {
    const xsrfToken = getCookie('_xsrf');
    const check = 'all';
    try {
      const response = await fetch('/backend/check-auth', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken,
        },
        body: JSON.stringify(check),
      });

      const result = await response.json();
      console.log(result.auth);
      if (response.ok && result.auth) {
        console.log('Authorized');
      } else {
        navigate('/unauth');
      }
    } catch (error) {
      console.error('Error setting XSRF token:', error);
    }
  };

  // useEffect hook to fetch XSRF token and check authorization when the component mounts
  useEffect(() => {
    fetchXsrfToken();
    CheckAuth();
  }, []);

  // Function to handle form submission
  const handleSubmit = async (e) => {
    e.preventDefault();

    // Confirmation message before submitting the form
    const confirmMessage = `
      Please confirm the following details:
      Lecture Title: ${formData.title}
      Module Number: ${formData.module_number}
      Semester: ${formData.semester}
      Description: ${formData.description}
      Start Date: ${formData.startDate}
      End Date: ${formData.endDate}
      Minimum Seats: ${formData.minSeats}
      Maximum Seats: ${formData.maxSeats}
      
      Do you want to submit?
    `;

    if (!window.confirm(confirmMessage)) {
      return;
    }

    console.log('Form submitted:', formData);

    try {
      const xsrfToken = getCookie('_xsrf');
      if (!xsrfToken) {
        console.error('XSRF token not found');
        alert('XSRF token not found. Please try again.');
        return;
      }
      console.log(`XSRF token: ${xsrfToken}`);

      const response = await fetch('/backend/create-db-lecture', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken,
        },
        body: JSON.stringify(formData),
      });

      if (!response.ok) {
        const errorText = await response.text();
        throw new Error(`Network response was not ok: ${errorText}`);
      }

      const result = await response.json();
      if (response.ok && result.success) {
        setMessage(result.message);
      } else {
        setMessage(result.message);
      }
    } catch (error) {
      console.error('There was a problem with the fetch operation:', error);
    }
  };

  return (
    <div>
      <div className="container mt-4">
        <h1>Add new course:</h1>
        <form onSubmit={handleSubmit} className="mt-4">
          <div className="mb-3">
            <label htmlFor="title" className="form-label">Lecture Title</label>
            <input
              type="text"
              className="form-control"
              id="title"
              name="title"
              value={formData.title}
              onChange={handleChange}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="module_number" className="form-label">Module Number (e.g., 1234-321)</label>
            <input
              type="text"
              className="form-control"
              id="module_number"
              name="module_number"
              value={formData.module_number}
              onChange={handleChange}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="semester" className="form-label">Semester (e.g., WS24 or SS24)</label>
            <input
              type="text"
              className="form-control"
              id="semester"
              name="semester"
              value={formData.semester}
              onChange={handleChange}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="description" className="form-label">Description</label>
            <textarea
              className="form-control"
              id="description"
              name="description"
              value={formData.description}
              onChange={handleChange}
            ></textarea>
          </div>
          <div className="mb-3">
            <label htmlFor="startDate" className="form-label">Start Date</label>
            <input
              type="date"
              className="form-control"
              id="startDate"
              name="startDate"
              value={formData.startDate}
              onChange={handleChange}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="endDate" className="form-label">End Date</label>
            <input
              type="date"
              className="form-control"
              id="endDate"
              name="endDate"
              value={formData.endDate}
              onChange={handleChange}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="minSeats" className="form-label">Minimum Seats</label>
            <input
              type="number"
              className="form-control"
              id="minSeats"
              name="minSeats"
              value={formData.minSeats}
              onChange={handleChange}
            />
          </div>
          <div className="mb-3">
            <label htmlFor="maxSeats" className="form-label">Maximum Seats</label>
            <input
              type="number"
              className="form-control"
              id="maxSeats"
              name="maxSeats"
              value={formData.maxSeats}
              onChange={handleChange}
            />
          </div>
          <button type="submit" className="btn btn-primary">Submit</button>
        </form>
        {message && <p className="mt-4">{message}</p>}
      </div>
    </div>
  );
};

export default FormComponent;
