import React, { useState, useEffect } from 'react';
import { Link, useNavigate, useLocation } from 'react-router-dom';

const Header = () => {
  // State to hold the name and role of the user
  const [name, setName] = useState('');
  const [role, setRole] = useState('');
  const navigate = useNavigate();
  const location = useLocation();
  const isAuthPage = location.pathname === '/login' || location.pathname === '/register';

  // Function to get a specific cookie value
  function getCookie(name) {
    const value = `; ${document.cookie}`;
    const parts = value.split(`; ${name}=`);
    if (parts.length === 2) return parts.pop().split(';').shift();
    return null;
  }

  // useEffect hook to fetch user information when the component mounts or location changes
  useEffect(() => {
    const fetchUserInfo = async () => {
      try {
        const response = await fetch('/backend/lecturer/username', {
          method: 'GET',
          credentials: 'include',
        });

        if (!response.ok) {
          const errorText = await response.text();
          throw new Error(`Network response was not ok: ${errorText}`);
        }

        const data = await response.json();
        console.log('API Response:', data);
        if (data) {
          if (data.name) {
            setName(data.name);
            console.log('SetName', data.name);
          }
          if (data.role) {
            setRole(data.role);
            console.log('SetRole', data.role);
          }
        } else {
          console.error('User data not found in the response');
        }
      } catch (error) {
        console.error('There was a problem with the fetch operation:', error);
      }
    };
    fetchUserInfo();
  }, [location.pathname]);

  // Function to handle user logout
  const handleLogout = () => {
    const xsrfToken = getCookie('_xsrf');
    console.log('xsrf Token: ', xsrfToken);
    fetch('/backend/lecturer/logout-handler', {
      method: 'GET',
      credentials: 'include',
      headers: {
        'Content-Type': 'application/json',
        'X-XSRF-TOKEN': xsrfToken,
      },
    })
      .then(response => {
        if (response.ok) {
          navigate('/login');
        } else {
          throw new Error('Logout failed');
        }
      })
      .catch(error => {
        console.error('There was an error logging out!', error);
      });
  };

  return (
    <nav className="navbar navbar-expand-lg navbar-dark bg-primary">
      <div className="container-fluid">
        {!isAuthPage && (
          <div className="collapse navbar-collapse">
            <ul className="navbar-nav me-auto">
              <li className="nav-item">
                <Link className="nav-link" to="/form">
                  Add course
                </Link>
              </li>
              <li className="nav-item">
                <Link className="nav-link" to="/yourcourses">
                  Your courses
                </Link>
              </li>
              <li className="nav-item">
                <Link className="nav-link" to="/account">
                  Account
                </Link>
              </li>
              {role === 'admin' && (
                <>
                  <li className="nav-item">
                    <Link className="nav-link" to="/admin/courses">
                      FIT Courses
                    </Link>
                  </li>
                  <li className="nav-item">
                    <Link className="nav-link" to="/admin/upload">
                      Upload courses
                    </Link>
                  </li>
                </>
              )}
            </ul>
            <span className="navbar-text me-3">
              You are logged in as {name}
            </span>
            <button className="btn btn-outline-light" onClick={handleLogout}>
              Logout
            </button>
          </div>
        )}
      </div>
    </nav>
  );
};

export default Header;
