import React, { useEffect, useState } from 'react';
import { Link, useNavigate } from 'react-router-dom';
import 'bootstrap/dist/css/bootstrap.min.css';

const Register = () => {
  const navigate = useNavigate();
  const [email, setEmail] = useState('');
  const [name, setName] = useState('');
  const [password, setPassword] = useState('');
  const [response, setResponse] = useState(null);
  const [error, setError] = useState(null);

  // Function to get a specific cookie by name
  function getCookie(name) {
    const value = `; ${document.cookie}`;
    const parts = value.split(`; ${name}=`);
    if (parts.length === 2) return parts.pop().split(';').shift();
    return null;
  }

  // useEffect hook to fetch XSRF token when the page loads
  useEffect(() => {
    const fetchXsrfToken = async () => {
      try {
        const response = await fetch('/backend/set-xsrf-token', {
          method: 'GET',
          credentials: 'include', // important to include cookies
        });

        if (!response.ok) {
          throw new Error('Failed to set XSRF token');
        }

        console.log('XSRF token set');
      } catch (error) {
        console.error('Error setting XSRF token:', error);
      }
    };

    fetchXsrfToken();
  }, []);

  // Function to handle registration
  const handleRegister = async () => {
    // Get XSRF token from cookies
    const xsrfToken = getCookie('_xsrf');
    if (!xsrfToken) {
      console.error('XSRF token not found');
      alert('XSRF token not found. Please try again.');
      return;
    }
    console.log(`XSRF token: ${xsrfToken}`);

    // Send POST request to backend
    const response = await fetch('/backend/lecturer/create_lecturer', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'X-XSRFToken': xsrfToken,
      },
      body: JSON.stringify({ email, name, password }),
    });

    // Read response from server
    const result = await response.json();
    console.log(result);
    if (response.ok && result.success) {
      navigate('/login');
    } else {
      setError(result.message);
    }
  };

  return (
    <div className="container mt-5">
      <h2>Register</h2>
      <form>
        <div className="mb-3">
          <label htmlFor="email" className="form-label">Email</label>
          <input
            type="text"
            className="form-control"
            id="email"
            value={email}
            onChange={(e) => setEmail(e.target.value)}
          />
        </div>
        <div className="mb-3">
          <label htmlFor="name" className="form-label">Name</label>
          <input
            type="text"
            className="form-control"
            id="name"
            value={name}
            onChange={(e) => setName(e.target.value)}
          />
        </div>
        <div className="mb-3">
          <label htmlFor="password" className="form-label">Password</label>
          <input
            type="password"
            className="form-control"
            id="password"
            value={password}
            onChange={(e) => setPassword(e.target.value)}
          />
        </div>
        <button type="button" className="btn btn-primary" onClick={handleRegister}>
          Register
        </button>
      </form>
      <p>Already have an account? <Link to="/login">Login here</Link></p>
      {error && <div className="alert alert-danger mt-3">{error}</div>}
    </div>
  );
};

export default Register;
