import React, { useEffect, useState } from 'react';
import 'bootstrap/dist/css/bootstrap.min.css';
import { useNavigate } from 'react-router-dom';

const CourseTable = () => {
  const [courses, setCourses] = useState([]);
  const [selectedCourses, setSelectedCourses] = useState([]);
  const [error, setError] = useState(null);
  const [notification, setNotification] = useState(null);
  const [message, setMessage] = useState(null);
  const navigate = useNavigate();

  function getCookie(name) {
    const value = `; ${document.cookie}`;
    const parts = value.split(`; ${name}=`);
    if (parts.length === 2) return parts.pop().split(';').shift();
    return null;
  }

  const fetchCourses = async () => {
    try {
      const response = await fetch('/backend/get-lectures-to-upload', {
        method: 'GET',
        credentials: 'include',
      });

      if (!response.ok) {
        const errorText = await response.text();
        throw new Error(`Network response was not ok: ${errorText}`);
      }

      const data = await response.json();
      if (data.message === "No lectures with NULL identifier found") {
        setMessage("No available courses found.");
      } else {
        setCourses(data.lectures);
        setMessage(null);
      }
    } catch (error) {
      console.error('There was a problem with the fetch operation:', error);
      setError('There was a problem with the fetch operation');
    }
  };
  const CheckAuth = async () => {
    const xsrfToken = getCookie('_xsrf');
    const check = 'admin'
    try {
      const response = await fetch('/backend/check-auth', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken,
        },
        body: JSON.stringify(check),
      });

      const result = await response.json();
      console.log(result.auth)
      if (response.ok && result.auth){
        console.log('Authorized')
      } else {
        navigate('/unauth');
      }
    } catch (error) {
      console.error('Error setting XSRF token:', error);
    }
  };
  useEffect(() => {
    CheckAuth();
    fetchCourses();
  }, []);

  const toggleCourse = (courseId) => {
    setSelectedCourses(prevSelected => {
      if (prevSelected.includes(courseId)) {
        return prevSelected.filter(id => id !== courseId);
      } else {
        return [...prevSelected, courseId];
      }
    });
  };

  const uploadSelectedCourses = async () => {
    try {
      const xsrfToken = getCookie('_xsrf');
      const response = await fetch('/backend/upload-lecture-to-fit', {
        method: 'POST',
        credentials: 'include',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken
        },
        body: JSON.stringify({ selectedCourses }),
      });

      if (!response.ok) {
        const errorText = await response.text();
        throw new Error(`Network response was not ok: ${errorText}`);
      }

      const result = await response.json();
      if (result.success) {
        setNotification(result.message || "Upload successful");
        setError(null);
        setSelectedCourses([]);
        fetchCourses();  // Refresh the courses list after successful upload
      } else {
        setError(result.message || "An error has occurred");
        setNotification(null);
      }
    } catch (error) {
      setError(error.message || "An error has occurred");
      setNotification(null);
    }
  };

  return (
    <div className="container mt-4">
      <h1 className="mb-4">Courses to upload: Overview</h1>
      {courses.length > 0 ? (
        <div className="table-responsive">
          <table className="table table-hover">
            <thead className="thead-dark">
              <tr>
                <th>Auswählen</th>
                <th>Titel</th>
                <th>Beschreibung</th>
              </tr>
            </thead>
            <tbody>
              {courses.map(course => (
                <tr key={course.id}>
                  <td>
                    <input
                      type="checkbox"
                      className="form-check-input"
                      checked={selectedCourses.includes(course.id)}
                      onChange={() => toggleCourse(course.id)}
                    />
                  </td>
                  <td>{course.title}</td>
                  <td>{course.description}</td>
                </tr>
              ))}
            </tbody>
          </table>
          {message && <div className="alert alert-info">{message}</div>}
        </div>
      ) : (
        !message && <div className="alert alert-info">Lade Kurse...</div>
      )}
      <button className="btn btn-primary mt-3" onClick={uploadSelectedCourses}>Upload</button>
      {notification && <div className="alert alert-success">{notification}</div>}
      {error && <div className="alert alert-danger">{error}</div>}

    </div>
  );
};

export default CourseTable;
