import React, { useEffect, useState } from 'react';
import { Link, useNavigate } from 'react-router-dom';
import 'bootstrap/dist/css/bootstrap.min.css';

const CourseTable = () => {
  // State to hold courses, error messages, notifications, and general messages
  const [courses, setCourses] = useState([]);
  const [error, setError] = useState(null);
  const [notification, setNotification] = useState(null);
  const [message, setMessage] = useState(null);
  const navigate = useNavigate();

  // Function to get a specific cookie by name
  function getCookie(name) {
    const value = `; ${document.cookie}`;
    const parts = value.split(`; ${name}=`);
    if (parts.length === 2) return parts.pop().split(';').shift();
    return null;
  }

  // Function to fetch courses from the backend
  const fetchCourses = async () => {
    try {
      const response = await fetch('/backend/get-courses-of-lecturer', {
        method: 'GET',
        credentials: 'include',
      });

      if (!response.ok) {
        const errorText = await response.text();
        throw new Error(`Network response was not ok: ${errorText}`);
      }

      const data = await response.json();
      console.log('Fetched data:', data);

      if (data.message === "No lectures") {
        setMessage("No available courses found.");
        setCourses([]);  // Add an example course
      } else {
        setCourses(data.lectures);
        setMessage(null);
      }
    } catch (error) {
      console.error('There was a problem with the fetch operation:', error);
      setError('There was a problem with the fetch operation');
    }
  };

  // Function to check if the user is authorized
  const CheckAuth = async () => {
    const xsrfToken = getCookie('_xsrf');
    const check = 'all';
    try {
      const response = await fetch('/backend/check-auth', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-XSRFToken': xsrfToken,
        },
        body: JSON.stringify(check),
      });

      const result = await response.json();
      console.log(result.auth);
      if (response.ok && result.auth) {
        console.log('Authorized');
      } else {
        navigate('/unauth');
      }
    } catch (error) {
      console.error('Error setting XSRF token:', error);
    }
  };

  // useEffect hook to check authorization and fetch courses when the component mounts
  useEffect(() => {
    CheckAuth();
    fetchCourses();
  }, []);

  return (
    <div className="container mt-4">
      <h1 className="mb-4">Your Courses: Overview</h1>
      {message && <div className="alert alert-info">{message}</div>}
      {courses.length === 0 && !message && !error && (
        <div className="alert alert-info">Loading courses...</div>
      )}
      {courses.length > 0 && (
        <div className="table-responsive">
          <table className="table table-hover">
            <thead className="thead-dark">
              <tr>
                <th>Title</th>
                <th>Description</th>
              </tr>
            </thead>
            <tbody>
              {courses.map(course => (
                <tr key={course.id}>
                  <td>{course.title}</td>
                  <td>{course.description}</td>
                </tr>
              ))}
            </tbody>
          </table>
        </div>
      )}
      <Link className="btn btn-primary mt-3" to="/form">Go to Form</Link>
      {notification && <div className="alert alert-success">{notification}</div>}
      {error && <div className="alert alert-danger">{error}</div>}
    </div>
  );
};

export default CourseTable;
